"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeEventListener = exports.updateEventListener = exports.removeAttribute = exports.updateAttribute = exports.createDomElement = exports.getEventFromAttr = exports.isEventAttr = void 0;
var isFunction_1 = require("../helpers/isFunction");
var EVENT_REGEXP = /^on[A-Z]/;
var EVENT_PREFIX_LENGTH = 2;
exports.isEventAttr = function (attrName) { return EVENT_REGEXP.test(attrName); };
exports.getEventFromAttr = function (attrName) { return attrName.substring(EVENT_PREFIX_LENGTH).toLowerCase(); };
function createDomElement(type, xmlns) {
    if (xmlns) {
        return document.createElementNS(xmlns, type);
    }
    else {
        return document.createElement(type);
    }
}
exports.createDomElement = createDomElement;
function updateAttribute(htmlElement, key, value) {
    if (value != null) {
        setAttribute(htmlElement, key, value);
    }
    else {
        removeAttribute(htmlElement, key);
    }
}
exports.updateAttribute = updateAttribute;
function setAttribute(htmlElement, key, value) {
    htmlElement.setAttribute(key, value);
}
function removeAttribute(htmlElement, key) {
    htmlElement.removeAttribute(key);
}
exports.removeAttribute = removeAttribute;
function updateEventListener(htmlElement, eventName, prev, curr) {
    removeEventListener(htmlElement, eventName, prev);
    addEventListener(htmlElement, eventName, curr);
}
exports.updateEventListener = updateEventListener;
// updates DOMNode props, compared to prevProps
function addEventListener(htmlElement, eventName, handler) {
    if (isFunction_1.isFunction(handler)) {
        htmlElement.addEventListener(eventName, handler);
    }
}
function removeEventListener(htmlElement, eventName, handler) {
    if (isFunction_1.isFunction(handler)) {
        htmlElement.removeEventListener(eventName, handler);
    }
}
exports.removeEventListener = removeEventListener;
