"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateDomChildNodesPipe = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var Static_1 = require("../engine/render/Static");
var isArray_1 = require("../helpers/isArray");
function updateDomChildNodesPipe(target) {
    var initialState = [];
    return rxjs_1.pipe(operators_1.startWith(initialState), operators_1.pairwise(), operators_1.tap(function (_a) {
        var e_1, _b;
        var _c = tslib_1.__read(_a, 2), prevChildren = _c[0], currChildren = _c[1];
        var elementsThatMount = [];
        // intial render
        // creates a fragment and appends the fragment to parent
        if (!prevChildren || !prevChildren.length) {
            var fragment_1 = window.document.createDocumentFragment();
            try {
                for (var currChildren_1 = tslib_1.__values(currChildren), currChildren_1_1 = currChildren_1.next(); !currChildren_1_1.done; currChildren_1_1 = currChildren_1.next()) {
                    var currChild = currChildren_1_1.value;
                    if (currChild) {
                        if (isArray_1.isArray(currChild)) {
                            currChild.forEach(function (entry) {
                                if (entry) {
                                    fragment_1.appendChild(entry.renderElement.htmlElement);
                                    if (entry.renderElement.element$) {
                                        elementsThatMount.push(entry.renderElement);
                                    }
                                }
                            });
                        }
                        else {
                            fragment_1.appendChild(currChild.htmlElement);
                            if (Static_1.isHTMLRenderElement(currChild)) {
                                elementsThatMount.push(currChild);
                            }
                        }
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (currChildren_1_1 && !currChildren_1_1.done && (_b = currChildren_1.return)) _b.call(currChildren_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            target.appendChild(fragment_1);
        }
        else {
            // smart comparison and DOM update
            var insertBeforeNode = null;
            for (var i = currChildren.length - 1; i >= 0; i--) {
                var prevChild = prevChildren[i];
                var currChild = currChildren[i];
                if (prevChild == currChild) {
                    if (!isArray_1.isArray(currChild)) {
                        insertBeforeNode = currChild.htmlElement;
                    }
                    else if (currChild.length && currChild[0].renderElement.htmlElement) {
                        insertBeforeNode = currChild[0].renderElement.htmlElement;
                    }
                    continue;
                }
                if (!isArray_1.isArray(prevChild) && !isArray_1.isArray(currChild)) {
                    if (prevChild.htmlElement) {
                        if (currChild.htmlElement) {
                            target.replaceChild(currChild.htmlElement, prevChild.htmlElement);
                            if (Static_1.isHTMLRenderElement(currChild)) {
                                elementsThatMount.push(currChild);
                            }
                            insertBeforeNode = currChild.htmlElement;
                        }
                        else {
                            target.removeChild(prevChild.htmlElement);
                        }
                        continue;
                    }
                    if (currChild) {
                        target.insertBefore(currChild.htmlElement, insertBeforeNode);
                        insertBeforeNode = currChild.htmlElement;
                        continue;
                    }
                    continue;
                }
                // array updates (-_-)
                // prevChild is array
                // remove all prev array elements, and add current child
                if (isArray_1.isArray(prevChild) && !isArray_1.isArray(currChild)) {
                    prevChild.forEach(function (entry) {
                        target.removeChild(entry.renderElement.htmlElement);
                    });
                    target.insertBefore(currChild.htmlElement, insertBeforeNode);
                    insertBeforeNode = currChild.htmlElement;
                    continue;
                }
                // currChild is array
                // remove prev element, and add array elements
                if (!isArray_1.isArray(prevChild) && isArray_1.isArray(currChild)) {
                    target.removeChild(prevChild.htmlElement);
                    if (!currChild.length) {
                        continue;
                    }
                    var documentFragment = window.document.createDocumentFragment();
                    for (var j = 0; j < currChild.length; j++) {
                        documentFragment.appendChild(currChild[j].renderElement.htmlElement);
                        if (Static_1.isHTMLRenderElement(currChild[j].renderElement)) {
                            elementsThatMount.push(currChild[j].renderElement);
                        }
                    }
                    target.insertBefore(documentFragment, insertBeforeNode);
                    insertBeforeNode = currChild[0].renderElement.htmlElement;
                    continue;
                }
                // array to array update
                // console.warn('suboptimal array update');
                if (isArray_1.isArray(prevChild) && isArray_1.isArray(currChild)) {
                    // store current array keys
                    var currKeys = (new Array(currChild.length)).fill(void 0);
                    // Try early exit if all elements are equal
                    if (prevChild.length == currChild.length) {
                        var arraysAreEqual = true;
                        for (var j = currChild.length - 1; j >= 0; j--) {
                            if (arraysAreEqual
                                && (currChild[j].key !== prevChild[j].key
                                    || currChild[j].renderElement.htmlElement !== prevChild[j].renderElement.htmlElement)) {
                                arraysAreEqual = false;
                            }
                            // store current array keys
                            currKeys[j] = currChild[j].key;
                        }
                        // early exit here
                        if (arraysAreEqual) {
                            continue;
                        }
                    }
                    else {
                        // store current array keys
                        for (var j = currChild.length - 1; j >= 0; j--) {
                            currKeys[j] = currChild[j].key;
                        }
                    }
                    var entriesThatStay = [];
                    for (var j = prevChild.length - 1; j >= 0; j--) {
                        if (!currKeys.includes(prevChild[j].key)) {
                            target.removeChild(prevChild[j].renderElement.htmlElement);
                        }
                        else {
                            entriesThatStay.unshift(prevChild[j]);
                        }
                    }
                    var k = entriesThatStay.length - 1;
                    for (var j = currChild.length - 1; j >= 0; j--) {
                        if (k >= 0 && currChild[j].key == entriesThatStay[k].key) {
                            if (currChild[j].renderElement.htmlElement !== entriesThatStay[k].renderElement.htmlElement) {
                                target.replaceChild(currChild[j].renderElement.htmlElement, entriesThatStay[k].renderElement.htmlElement);
                                if (Static_1.isHTMLRenderElement(currChild[j].renderElement)) {
                                    elementsThatMount.push(currChild[j].renderElement);
                                }
                            }
                            insertBeforeNode = currChild[j].renderElement.htmlElement;
                            k--;
                            continue;
                        }
                        try {
                            target.insertBefore(currChild[j].renderElement.htmlElement, insertBeforeNode);
                            if (Static_1.isHTMLRenderElement(currChild[j].renderElement)) {
                                elementsThatMount.push(currChild[j].renderElement);
                            }
                        }
                        catch (e) {
                            // TODO: atm there are cases when prev node
                            //       is absent on the DOM
                            //       possible reasons:
                            //       - it was replaced or wasnt added at all
                            //       - a key occures two or more times in the array
                            console.error(e);
                        }
                        insertBeforeNode = currChild[j].renderElement.htmlElement;
                    }
                    for (; k >= 0; k--) {
                        target.removeChild(entriesThatStay[k].renderElement.htmlElement);
                        if (Static_1.isHTMLRenderElement(entriesThatStay[k].renderElement)) {
                            elementsThatMount.push(entriesThatStay[k].renderElement);
                        }
                    }
                    continue;
                }
            }
        }
        elementsThatMount.forEach(function (el) { return el.element$.next(el.htmlElement); });
    }));
}
exports.updateDomChildNodesPipe = updateDomChildNodesPipe;
