"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicEntry = void 0;
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var component_1 = require("./component");
var Element_1 = require("./Element");
var helpers_1 = require("./component/helpers");
var NULL_CHILD_STUB = Object.create(null);
exports.DynamicEntry = function () {
    var update$ = new rxjs_1.Subject();
    var result$ = new rxjs_1.ReplaySubject(1);
    var destroy$ = new rxjs_1.Subject();
    var component;
    var prev = NULL_CHILD_STUB;
    destroy$.pipe(operators_1.take(1)).subscribe(function () {
        if (component) {
            component.destroy$.next(void 0);
        }
    });
    update$.pipe(operators_1.takeUntil(destroy$))
        .subscribe(function (curr) {
        // create a new component if:
        // - there were no prev value
        // - different data types of children
        // - different types of elements (leaf / array / static / observable / fn)
        // - keys mismatch
        // - xmlns mismatch
        // - number of children mismatch
        if (prev === NULL_CHILD_STUB
            || helpers_1.getType(prev) != helpers_1.getType(curr)
            || (Element_1.isElement(curr) && Element_1.isElement(prev) && (prev.type !== curr.type
                || !Object.is(prev.props.key, curr.props.key)
                || !Object.is(prev.props.xmlns, curr.props.xmlns)
                || prev.props.children.length !== curr.props.children.length))) {
            if (component) {
                component.destroy$.next(void 0);
            }
            component = component_1.createComponent(curr);
            result$.next(component);
        }
        component.update$.next(curr);
        prev = curr;
    });
    return {
        update$: update$,
        result$: result$.pipe(operators_1.takeUntil(destroy$)),
        destroy$: destroy$
    };
};
