"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createArrayComponent = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var DynamicEntry_1 = require("../DynamicEntry");
var helpers_1 = require("./helpers");
function createArrayComponent() {
    var update$ = new rxjs_1.Subject();
    var destroy$ = new rxjs_1.Subject();
    var items$ = new rxjs_1.ReplaySubject(1);
    var dynamicEntries = new Map();
    destroy$.pipe(operators_1.take(1)).subscribe(function () {
        var e_1, _a;
        try {
            for (var _b = tslib_1.__values(dynamicEntries.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var value = _c.value;
                value.destroy$.next(void 0);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
    update$.pipe(operators_1.startWith(null), operators_1.pairwise(), operators_1.switchMap(function (_a) {
        // NOTE: this code block is similar to Array Rendering logic
        // TODO: refactor
        var _b = tslib_1.__read(_a, 2), prev = _b[0], curr = _b[1];
        // shortcut
        // if curr array is empty -- just return empty array
        if (curr.length == 0) {
            dynamicEntries.clear();
            return rxjs_1.of([]);
        }
        // shortcut
        // if all elements (keys) are the same -- just push an update to them
        if (prev && prev.length == curr.length && prev.every(function (p, i) { return p.props.key == curr[i].props.key; })) {
            curr.forEach(function (definition) {
                var key = definition.props.key;
                if (key == null) {
                    throw 'Key should be defined';
                }
                dynamicEntries.get(key).update$.next(definition);
            });
            return rxjs_1.EMPTY;
        }
        // removing obsolete keys
        if (prev && prev.length != 0) {
            for (var prevIndex = 0; prevIndex < prev.length; prevIndex++) {
                var shouldRemove = true;
                var prevKey = prev[prevIndex].props.key;
                for (var currKey = 0; currKey < curr.length; currKey++) {
                    if (prevKey == curr[currKey].props.key) {
                        shouldRemove = false;
                        break;
                    }
                }
                if (shouldRemove) {
                    var dynamicEntry = dynamicEntries.get(prevKey);
                    dynamicEntry.destroy$.next(void 0);
                    dynamicEntries.delete(prevKey);
                }
            }
        }
        return rxjs_1.combineLatest.apply(void 0, tslib_1.__spread(curr.map(function (definition) {
            var key = definition.props.key;
            if (key == null || (typeof key !== 'string' && typeof key !== 'number')) {
                console.error(key);
                throw 'Key should be string or number';
            }
            if (!dynamicEntries.has(key)) {
                dynamicEntries.set(key, DynamicEntry_1.DynamicEntry());
            }
            var dynamicEntry = dynamicEntries.get(key);
            dynamicEntry.update$.next(definition);
            return dynamicEntry.result$.pipe(operators_1.map(function (component) { return ({ key: key, component: component }); }));
        })));
    }), operators_1.takeUntil(destroy$)).subscribe(items$);
    return {
        type: helpers_1.ComponentType.array,
        items$: items$.asObservable(),
        update$: update$,
        destroy$: destroy$
    };
}
exports.createArrayComponent = createArrayComponent;
