"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFnComponent = void 0;
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var DynamicEntry_1 = require("../DynamicEntry");
var isThenable_1 = require("../../helpers/isThenable");
var helpers_1 = require("./helpers");
function createFnComponent(definition) {
    var update$ = new rxjs_1.Subject();
    var destroy$ = new rxjs_1.Subject();
    var props$ = update$.pipe(operators_1.map(function (update) { return update.props; }), operators_1.takeUntil(destroy$));
    var dynamicRoot = DynamicEntry_1.DynamicEntry();
    var destroyTake1$ = destroy$.pipe(operators_1.take(1));
    destroyTake1$
        .subscribe(dynamicRoot.destroy$);
    var result = definition.type(props$, { destroy$: destroyTake1$ });
    asObservable(result)
        .pipe(operators_1.takeUntil(destroy$))
        .subscribe(dynamicRoot.update$);
    return {
        type: helpers_1.ComponentType.fn,
        update$: update$,
        destroy$: destroy$,
        result$: dynamicRoot.result$,
    };
}
exports.createFnComponent = createFnComponent;
function asObservable(value) {
    if (rxjs_1.isObservable(value)) {
        return value;
    }
    // basically RxJS way of checking against Promise
    // https://github.com/ReactiveX/rxjs/blob/8ad3267fb337a22c383a4ff8901e03b0ae3e2a7a/src/internal/util/isPromise.ts
    if (value && isThenable_1.isThenable(value)) {
        return rxjs_1.from(value);
    }
    // TODO: check if the value can be rendered
    // (is an Element or a basic type)
    // throw otherwise
    return rxjs_1.of(value);
}
