"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderStatic = exports.isHTMLRenderElement = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("../../constants");
var DomElement_1 = require("../../dom/DomElement");
var UpdateDomChildNodesPipe_1 = require("../../dom/UpdateDomChildNodesPipe");
var isSubject_1 = require("../../helpers/isSubject");
var index_1 = require("./index");
exports.isHTMLRenderElement = function (element) {
    return element && 'type' in element && element.type == 'Element';
};
// watch updates
//    update domElement
// subscribe to child updates
//    for each child udpate -- render component( target = domElement )
function renderStatic(component, xmlns) {
    // xmlns is always the same for the same component
    // due to differentiation in DynamicEntry
    xmlns = component.definition.props.xmlns || xmlns;
    if (component.definition.type === 'svg' && !component.definition.props.xmlns) {
        xmlns = "http://www.w3.org/2000/svg";
    }
    var htmlElement = DomElement_1.createDomElement(component.definition.type, xmlns);
    // UPDATE DOM ELEMENT
    component.change$.pipe(splitPropsToStreams(), 
    // subscribe to each stream and update DOM accordingly
    operators_1.flatMap(function (group) {
        var key = group.key;
        if (constants_1.PRESERVED_KEYS.includes(group.key)) {
            return rxjs_1.EMPTY;
        }
        var distinct$ = group.pipe(operators_1.distinctUntilChanged(), operators_1.startWith(void 0), operators_1.share());
        var isEvent = DomElement_1.isEventAttr(group.key);
        if (!isEvent) {
            return distinct$.pipe(operators_1.tap({
                next: function (value) { return DomElement_1.updateAttribute(htmlElement, key, value); },
                complete: function () { return DomElement_1.removeAttribute(htmlElement, key); }
            }));
        }
        else {
            var eventName_1 = isEvent && DomElement_1.getEventFromAttr(key);
            var latestValue_1;
            return distinct$.pipe(operators_1.map(function (value) { return isSubject_1.isSubject(value) ? function (x) { return value.next(x); } : value; }), operators_1.pairwise(), operators_1.tap({
                next: function (_a) {
                    var _b = tslib_1.__read(_a, 2), a = _b[0], b = _b[1];
                    latestValue_1 = b;
                    DomElement_1.updateEventListener(htmlElement, eventName_1, a, b);
                },
                complete: function () {
                    DomElement_1.removeEventListener(htmlElement, eventName_1, latestValue_1);
                }
            }));
        }
    })).subscribe();
    var childrenXmlns = component.definition.type === 'foreignObject'
        ? null
        : xmlns;
    // NOTE: perf optimisation:
    // to make first render faster, we wait for all children to emit their
    // first value (vDOM) and only then we append all emissions to the parent
    rxjs_1.combineLatest.apply(void 0, tslib_1.__spread(component.dynamicChildren
        .map(function (dynamicChild) {
        return dynamicChild.result$.pipe(operators_1.switchMap(function (result) { return index_1.renderComponent(result, childrenXmlns); }));
    }))).pipe(UpdateDomChildNodesPipe_1.updateDomChildNodesPipe(htmlElement))
        .subscribe();
    return rxjs_1.of({
        type: 'Element',
        element$: component.element$,
        htmlElement: htmlElement
    });
}
exports.renderStatic = renderStatic;
// split change$ stream into individual prop streams
function splitPropsToStreams() {
    return function (source$) { return new rxjs_1.Observable(function (observer) {
        var map = new Map();
        var subscription = source$.subscribe({
            next: function (change) {
                var e_1, _a, e_2, _b;
                var changeEntries = Object.entries(change);
                try {
                    for (var changeEntries_1 = tslib_1.__values(changeEntries), changeEntries_1_1 = changeEntries_1.next(); !changeEntries_1_1.done; changeEntries_1_1 = changeEntries_1.next()) {
                        var _c = tslib_1.__read(changeEntries_1_1.value, 2), key = _c[0], value = _c[1];
                        var stream = void 0;
                        if (!map.has(key)) {
                            stream = new rxjs_1.Subject();
                            stream['key'] = key;
                            map.set(key, stream);
                            observer.next(stream);
                        }
                        if (!stream) {
                            stream = map.get(key);
                        }
                        stream.next(value);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (changeEntries_1_1 && !changeEntries_1_1.done && (_a = changeEntries_1.return)) _a.call(changeEntries_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                try {
                    for (var _d = tslib_1.__values(map.keys()), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var oldKey = _e.value;
                        if (oldKey in change) {
                            continue;
                        }
                        var deprecatedStream = map.get(oldKey);
                        deprecatedStream.complete();
                        map.delete(oldKey);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_e && !_e.done && (_b = _d.return)) _b.call(_d);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            },
            error: observer.error,
            complete: observer.complete
        });
        // complete all streams on unsubscription
        subscription.add(function () {
            tslib_1.__spread(map.values()).forEach(function (v) { return v.complete(); });
        });
        return subscription;
    }); };
}
