"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderComponent = exports.render = void 0;
var operators_1 = require("rxjs/operators");
var UpdateDomChildNodesPipe_1 = require("../../dom/UpdateDomChildNodesPipe");
var component_1 = require("../component");
var Array_1 = require("./Array");
var Fn_1 = require("./Fn");
var Leaf_1 = require("./Leaf");
var Observable_1 = require("./Observable");
var Static_1 = require("./Static");
exports.render = function (definition, target) {
    var root = component_1.createComponent(definition);
    var subscription = renderRootComponent(root, target)
        .subscribe();
    root.update$.next(definition);
    // TODO: add memory leak test
    //       unsubscription might not spread up the tree
    return subscription;
};
function renderRootComponent(component, target) {
    return renderComponent(component, null).pipe(operators_1.map(function (el) { return [el]; }), UpdateDomChildNodesPipe_1.updateDomChildNodesPipe(target));
}
function renderComponent(component, xmlns) {
    if (component.type == component_1.ComponentType.leaf) {
        return Leaf_1.renderLeaf(component);
    }
    else if (component.type == component_1.ComponentType.fn) {
        return Fn_1.renderFn(component, xmlns);
    }
    else if (component.type == component_1.ComponentType.static) {
        return Static_1.renderStatic(component, xmlns);
    }
    else if (component.type == component_1.ComponentType.observable) {
        return Observable_1.renderObservable(component, xmlns);
    }
    else if (component.type == component_1.ComponentType.array) {
        return Array_1.renderArray(component, xmlns);
    }
    throw 'Unknown component type';
}
exports.renderComponent = renderComponent;
