import { isFunction } from '../helpers/isFunction';
const EVENT_REGEXP = /^on[A-Z]/;
const EVENT_PREFIX_LENGTH = 2;
export const isEventAttr = (attrName) => EVENT_REGEXP.test(attrName);
export const getEventFromAttr = (attrName) => attrName.substring(EVENT_PREFIX_LENGTH).toLowerCase();
export function createDomElement(type, xmlns) {
    if (xmlns) {
        return document.createElementNS(xmlns, type);
    }
    else {
        return document.createElement(type);
    }
}
export function updateAttribute(htmlElement, key, value) {
    if (value != null) {
        setAttribute(htmlElement, key, value);
    }
    else {
        removeAttribute(htmlElement, key);
    }
}
function setAttribute(htmlElement, key, value) {
    htmlElement.setAttribute(key, value);
}
export function removeAttribute(htmlElement, key) {
    htmlElement.removeAttribute(key);
}
export function updateEventListener(htmlElement, eventName, prev, curr) {
    removeEventListener(htmlElement, eventName, prev);
    addEventListener(htmlElement, eventName, curr);
}
// updates DOMNode props, compared to prevProps
function addEventListener(htmlElement, eventName, handler) {
    if (isFunction(handler)) {
        htmlElement.addEventListener(eventName, handler);
    }
}
export function removeEventListener(htmlElement, eventName, handler) {
    if (isFunction(handler)) {
        htmlElement.removeEventListener(eventName, handler);
    }
}
