import { __read, __spread, __values } from "tslib";
import { combineLatest, EMPTY, of, ReplaySubject, Subject } from 'rxjs';
import { map, pairwise, startWith, switchMap, take, takeUntil } from 'rxjs/operators';
import { DynamicEntry } from '../DynamicEntry';
import { ComponentType } from './helpers';
export function createArrayComponent() {
    var update$ = new Subject();
    var destroy$ = new Subject();
    var items$ = new ReplaySubject(1);
    var dynamicEntries = new Map();
    destroy$.pipe(take(1)).subscribe(function () {
        var e_1, _a;
        try {
            for (var _b = __values(dynamicEntries.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var value = _c.value;
                value.destroy$.next(void 0);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
    update$.pipe(startWith(null), pairwise(), switchMap(function (_a) {
        // NOTE: this code block is similar to Array Rendering logic
        // TODO: refactor
        var _b = __read(_a, 2), prev = _b[0], curr = _b[1];
        // shortcut
        // if curr array is empty -- just return empty array
        if (curr.length == 0) {
            dynamicEntries.clear();
            return of([]);
        }
        // shortcut
        // if all elements (keys) are the same -- just push an update to them
        if (prev && prev.length == curr.length && prev.every(function (p, i) { return p.props.key == curr[i].props.key; })) {
            curr.forEach(function (definition) {
                var key = definition.props.key;
                if (key == null) {
                    throw 'Key should be defined';
                }
                dynamicEntries.get(key).update$.next(definition);
            });
            return EMPTY;
        }
        // removing obsolete keys
        if (prev && prev.length != 0) {
            for (var prevIndex = 0; prevIndex < prev.length; prevIndex++) {
                var shouldRemove = true;
                var prevKey = prev[prevIndex].props.key;
                for (var currKey = 0; currKey < curr.length; currKey++) {
                    if (prevKey == curr[currKey].props.key) {
                        shouldRemove = false;
                        break;
                    }
                }
                if (shouldRemove) {
                    var dynamicEntry = dynamicEntries.get(prevKey);
                    dynamicEntry.destroy$.next(void 0);
                    dynamicEntries.delete(prevKey);
                }
            }
        }
        return combineLatest.apply(void 0, __spread(curr.map(function (definition) {
            var key = definition.props.key;
            if (key == null || (typeof key !== 'string' && typeof key !== 'number')) {
                console.error(key);
                throw 'Key should be string or number';
            }
            if (!dynamicEntries.has(key)) {
                dynamicEntries.set(key, DynamicEntry());
            }
            var dynamicEntry = dynamicEntries.get(key);
            dynamicEntry.update$.next(definition);
            return dynamicEntry.result$.pipe(map(function (component) { return ({ key: key, component: component }); }));
        })));
    }), takeUntil(destroy$)).subscribe(items$);
    return {
        type: ComponentType.array,
        items$: items$.asObservable(),
        update$: update$,
        destroy$: destroy$
    };
}
