import { __read, __spread, __values } from "tslib";
import { combineLatest, EMPTY, Observable, of, Subject } from 'rxjs';
import { distinctUntilChanged, flatMap, map, pairwise, share, startWith, switchMap, tap } from 'rxjs/operators';
import { PRESERVED_KEYS } from '../../constants';
import { createDomElement, getEventFromAttr, isEventAttr, removeAttribute, removeEventListener, updateAttribute, updateEventListener } from '../../dom/DomElement';
import { updateDomChildNodesPipe } from '../../dom/UpdateDomChildNodesPipe';
import { isSubject } from '../../helpers/isSubject';
import { renderComponent } from './index';
export var isHTMLRenderElement = function (element) {
    return element && 'type' in element && element.type == 'Element';
};
// watch updates
//    update domElement
// subscribe to child updates
//    for each child udpate -- render component( target = domElement )
export function renderStatic(component, xmlns) {
    // xmlns is always the same for the same component
    // due to differentiation in DynamicEntry
    xmlns = component.definition.props.xmlns || xmlns;
    if (component.definition.type === 'svg' && !component.definition.props.xmlns) {
        xmlns = "http://www.w3.org/2000/svg";
    }
    var htmlElement = createDomElement(component.definition.type, xmlns);
    // UPDATE DOM ELEMENT
    component.change$.pipe(splitPropsToStreams(), 
    // subscribe to each stream and update DOM accordingly
    flatMap(function (group) {
        var key = group.key;
        if (PRESERVED_KEYS.includes(group.key)) {
            return EMPTY;
        }
        var distinct$ = group.pipe(distinctUntilChanged(), startWith(void 0), share());
        var isEvent = isEventAttr(group.key);
        if (!isEvent) {
            return distinct$.pipe(tap({
                next: function (value) { return updateAttribute(htmlElement, key, value); },
                complete: function () { return removeAttribute(htmlElement, key); }
            }));
        }
        else {
            var eventName_1 = isEvent && getEventFromAttr(key);
            var latestValue_1;
            return distinct$.pipe(map(function (value) { return isSubject(value) ? function (x) { return value.next(x); } : value; }), pairwise(), tap({
                next: function (_a) {
                    var _b = __read(_a, 2), a = _b[0], b = _b[1];
                    latestValue_1 = b;
                    updateEventListener(htmlElement, eventName_1, a, b);
                },
                complete: function () {
                    removeEventListener(htmlElement, eventName_1, latestValue_1);
                }
            }));
        }
    })).subscribe();
    var childrenXmlns = component.definition.type === 'foreignObject'
        ? null
        : xmlns;
    // NOTE: perf optimisation:
    // to make first render faster, we wait for all children to emit their
    // first value (vDOM) and only then we append all emissions to the parent
    combineLatest.apply(void 0, __spread(component.dynamicChildren
        .map(function (dynamicChild) {
        return dynamicChild.result$.pipe(switchMap(function (result) { return renderComponent(result, childrenXmlns); }));
    }))).pipe(updateDomChildNodesPipe(htmlElement))
        .subscribe();
    return of({
        type: 'Element',
        element$: component.element$,
        htmlElement: htmlElement
    });
}
// split change$ stream into individual prop streams
function splitPropsToStreams() {
    return function (source$) { return new Observable(function (observer) {
        var map = new Map();
        var subscription = source$.subscribe({
            next: function (change) {
                var e_1, _a, e_2, _b;
                var changeEntries = Object.entries(change);
                try {
                    for (var changeEntries_1 = __values(changeEntries), changeEntries_1_1 = changeEntries_1.next(); !changeEntries_1_1.done; changeEntries_1_1 = changeEntries_1.next()) {
                        var _c = __read(changeEntries_1_1.value, 2), key = _c[0], value = _c[1];
                        var stream = void 0;
                        if (!map.has(key)) {
                            stream = new Subject();
                            stream['key'] = key;
                            map.set(key, stream);
                            observer.next(stream);
                        }
                        if (!stream) {
                            stream = map.get(key);
                        }
                        stream.next(value);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (changeEntries_1_1 && !changeEntries_1_1.done && (_a = changeEntries_1.return)) _a.call(changeEntries_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                try {
                    for (var _d = __values(map.keys()), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var oldKey = _e.value;
                        if (oldKey in change) {
                            continue;
                        }
                        var deprecatedStream = map.get(oldKey);
                        deprecatedStream.complete();
                        map.delete(oldKey);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_e && !_e.done && (_b = _d.return)) _b.call(_d);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            },
            error: observer.error,
            complete: observer.complete
        });
        // complete all streams on unsubscription
        subscription.add(function () {
            __spread(map.values()).forEach(function (v) { return v.complete(); });
        });
        return subscription;
    }); };
}
